<?php
/*
  Plugin Name: Expay Payment Gateway
  Plugin URI:
  Description: Allows you to use Expay gateway
  Version: 2.0.3
  Author: Expay Software Co., Ltd
  Text Domain: wc_expay_language
  Author URI: https://www.expay.asia
 */

defined('ABSPATH') or die('No direct script access allowed.');

add_filter('woocommerce_payment_gateways', 'expay_add_gateway_class');
function expay_add_gateway_class($gateways)
{
    $gateways[] = 'ExpayGateway';
    return $gateways;
}

add_action('plugins_loaded', 'expay_register_gateway_class');
function expay_register_gateway_class()
{
    class ExpayGateway extends WC_Payment_Gateway
    {

        private $provider;
        private $payment_categories;
        private $payment_methods;

        public function __construct()
        {
            $this->init_expay_settings();
            $this->include_classes();
            $this->session_init();

            $this->provider = new ExpayProvider();
            $this->payment_categories = $this->provider->getServicesCategories();
            $methodList = $this->provider->getMethodList();

            if (is_array($methodList)) {
                foreach ($methodList as $method) {
                    $this->payment_methods[$method['id']] = $method;
                }
            }
        }

        private function session_init()
        {
            WC()->session = new WC_Session_Handler();
            WC()->session->init();
            WC()->session->set('publishable_key', $this->publishable_key);
            WC()->session->set('private_key', $this->private_key);
            WC()->session->set('testmode', $this->testmode);
        }


        private function init_expay_settings()
        {
            $this->id = 'expay';
            $this->has_fields = true;
            $this->order_button_text = __('Pay via Expay', 'expay');
            $this->method_title = __('Expay Gateway Merchant API', 'expayApi');

            $this->init_form_fields();

            $this->init_settings();
            $this->title = $this->get_option('title');
            $this->description = $this->get_option('description');
            $this->enabled = $this->get_option('enabled');
            $this->testmode = 'yes' === $this->get_option('testmode');
            $this->private_key = $this->testmode ? $this->get_option('test_private_key') : $this->get_option('private_key');
            $this->publishable_key = $this->testmode ? $this->get_option('test_publishable_key') : $this->get_option('publishable_key');
            $this->instructions = $this->get_option('instructions');

            if (version_compare(WOOCOMMERCE_VERSION, '2.0.0', '>='))
                add_action('woocommerce_update_options_payment_gateways_' . $this->id, array(&$this, 'process_admin_options'));
            else
                add_action('woocommerce_update_options_payment_gateways', array(&$this, 'process_admin_options'));
            add_action('admin_notices', array($this, 'admin_notices'));

            add_action('woocommerce_receipt_' . $this->id, array($this, 'expay_waiting_form'));
            add_action('woocommerce_api_expay', array($this, 'webhook'), 10);
        }

        private function include_classes()
        {
            defined('EXPAY_WOOCOMMERCE_PLUGIN_PATH') || define('EXPAY_WOOCOMMERCE_PLUGIN_PATH', __DIR__);
            require_once EXPAY_WOOCOMMERCE_PLUGIN_PATH . '/includes/utils/ExpayProvider.php';
            require_once EXPAY_WOOCOMMERCE_PLUGIN_PATH . '/includes/utils/Curl.php';
            require_once EXPAY_WOOCOMMERCE_PLUGIN_PATH . '/includes/utils/ExpayPayment.php';
            require_once EXPAY_WOOCOMMERCE_PLUGIN_PATH . '/includes/events/ExpayEventHandler.php';
            require_once EXPAY_WOOCOMMERCE_PLUGIN_PATH . '/includes/events/ExpayBaseEvent.php';
        }

        public function admin_notices()
        {
            if ($this->enabled == 'no') {
                return false;
            }

            if (!$this->private_key && !$this->publishable_key) {
                echo '<div class="error"><p>' . __('You need the API Key & Secret Key in order to work, please find your API key and Secret at <a href="https://api.expay.asia/merchant/doc" target="_blank">Expay Gateway Authentication Keys</a>.', 'beanstream-for-woocommerce') . '</p></div>';
                return false;
            }
            return true;
        }

        public function init_form_fields()
        {

            $this->form_fields = array(
                'callback_url' => array(
                    'title' => 'Url for callback',
                    'type' => 'title',
                    'description' => 'Setup this URL as Callback URL: ' . str_replace('http:', 'https:', add_query_arg('wc-api', 'expay', home_url('/'))),
                ),
                'success_url' => array(
                    'title' => 'Success url',
                    'type' => 'title',
                    'description' => 'Setup this URL as Success URL: ' . str_replace('http:', 'https:', add_query_arg('wc-api', 'expay&status=success', home_url('/'))),
                ),
                'reject_url' => array(
                    'title' => 'Reject url',
                    'type' => 'title',
                    'description' => 'Setup this URL as Reject URL: ' . str_replace('http:', 'https:', add_query_arg('wc-api', 'expay&status=reject', home_url('/'))),
                ),
                'waiting_url' => array(
                    'title' => 'Waiting url',
                    'type' => 'title',
                    'description' => 'Setup this URL as Waiting URL: ' . str_replace('http:', 'https:', add_query_arg('wc-api', 'expay&status=waiting', home_url('/'))),
                ),
                'enabled' => array(
                    'title' => 'Enable/Disable',
                    'label' => 'Enable Expay Gateway',
                    'type' => 'checkbox',
                    'description' => '',
                    'default' => 'no'
                ),
                'title' => array(
                    'title' => 'Title',
                    'type' => 'text',
                    'default' => 'Expay',
                    'desc_tip' => true,
                ),
                'testmode' => array(
                    'title' => 'Test mode',
                    'label' => 'Enable Test Mode',
                    'type' => 'checkbox',
                    'description' => 'Place the payment gateway in test mode using test API keys.',
                    'default' => 'yes',
                    'desc_tip' => true,
                ),
                'test_publishable_key' => array(
                    'title' => 'Test Publishable Key',
                    'type' => 'text'
                ),
                'test_private_key' => array(
                    'title' => 'Test Private Key',
                    'type' => 'password',
                ),
                'publishable_key' => array(
                    'title' => 'Live Publishable Key',
                    'type' => 'text'
                ),
                'private_key' => array(
                    'title' => 'Live Private Key',
                    'type' => 'password'
                ),
            );
        }

        public function payment_fields()
        {
            echo '<script type="text/javascript" src="https://code.jquery.com/jquery-latest.min.js"></script>
                  <script src="' . plugin_dir_url(__FILE__) . 'assets/js/jquery_form.js"></script>
                  <script src="https://unpkg.com/sweetalert/dist/sweetalert.min.js"></script>
                  <link rel="stylesheet" type="text/css" href=" ' . plugin_dir_url(__FILE__) . 'assets/css/expay.css">
                  <script type="text/javascript" src="' . plugin_dir_url(__FILE__) . 'assets/js/expay.js"></script>';

            if (is_array($this->payment_categories)) {
                foreach ($this->payment_categories as $category) {
                    echo '<div><input type="radio" class="payment-category-radio" name="expay-payment-category" id="' . $category['id'] . '" value="' . $category['key'] . '"/><label class="category-title" for="' . $category['id'] . '">' . $category['name'] . ' </label> <fieldset id="' . $category['key'] . '" class="payment-field">';
                    foreach ($category['services'] as $service_id) {
                        if (isset($this->payment_methods[$service_id])) {
                            echo '<ul class="expay-banks-list">
                        <div class="item"><input class="payment-method-radio" id="' . $this->payment_methods[$service_id]['id'] . '" type="radio" name="expay-payment-method" value="' . $this->payment_methods[$service_id]['alias'] . '"/>';
                            if (isset($this->payment_methods[$service_id]['image'])) {
                                echo '<label for="' . $this->payment_methods[$service_id]['id'] . '">
                                  <div class="bank-logo"><img src="' . $this->payment_methods[$service_id]['image'] . '"></div>
                                  </label>';
                            }
                            echo '<div class="bank-label"><label for="' . $this->payment_methods[$service_id]['id'] . '"> ' . $this->payment_methods[$service_id]['name'] . '</label></div><div class="attribute-list">';
                            foreach ($this->payment_methods[$service_id]['attributes'] as $attribute) {
                                if ($attribute['required'] == true) {
                                    echo '<input class="attribute" type="text" name="' . $attribute['key'] . '"data-regexp="' . $attribute['regexp'] . '"placeholder="' . $attribute['description'] . '" required disabled/>';
                                } else {
                                    echo '<input class="attribute" type="text" name="' . $attribute['key'] . '"data-regexp="' . $attribute['regexp'] . '"placeholder="' . $attribute['description'] . '" disabled/>';
                                }
                            }
                            echo '</div></div></ul>';
                        }
                    }
                    echo '</fieldset></div>';
                }
            }
        }

        public function process_payment_error($message, $order_id)
        {
            $order = wc_get_order($order_id);
            error_log($message);
            $order->add_order_note($message);
            $order->update_status('cancelled', $message);
            wc_add_notice('Error in proccess payment. Please try again later.', 'error');
            /*return [
                'result' => 'success',
                'redirect' => $order->get_cancel_order_url()
            ];*/
        }

        public function process_payment($order_id)
        {
            global $woocommerce;

            $order = wc_get_order($order_id);
            $order_amount = $order->get_total();
            $order_number = $order->get_order_number();
            $expay_payment = new ExpayPayment($order->get_view_order_url());
            $payment_attempt_response = $expay_payment->initPayment($order_amount, $order_number, $this->payment_methods);

            if ((is_array($payment_attempt_response)) && ($payment_attempt_response['status'] === 206)) {
                $woocommerce->cart->empty_cart();
                $order->update_status('pending', 'Awaiting payment');
                $order->add_order_note('Awaiting payment');
                WC()->session->set('redirectUrl', $expay_payment->getPaymentUrl($payment_attempt_response));
                //error_log("INVOIOCE ID: " . $payment_attempt_response['id']);
                return [
                    'result' => 'success',
                    'redirect' => $order->get_checkout_payment_url(true)
                ];
            } elseif (isset($payment_attempt_response['error'])) {
                return $this->process_payment_error('Expay Error' . $payment_attempt_response['error']['message'] . '. Order id: ' . $order_id, $order_id);
            } elseif ($payment_attempt_response['status'] == 204) {
                return $this->process_payment_error("Expay Error: Payment status is " . $payment_attempt_response['status'] . "Order id: $order_id", $order_id);
            } elseif (is_null($payment_attempt_response)) {
                return $this->process_payment_error("Expay Error: Response from Expay server is empty", $order_id);
            }
        }

        public function expay_waiting_form($order_id)
        {
            $order = wc_get_order($order_id);
            $redirectUrl = WC()->session->get('redirectUrl', '');
            echo '<script type="text/javascript" src="https://code.jquery.com/jquery-latest.min.js"></script>
                  <script src="https://malsup.github.com/jquery.form.js"></script>
                  <link rel="stylesheet" type="text/css" href=" ' . plugin_dir_url(__FILE__) . 'assets/css/expay.css">
                  <script type="text/javascript" src="' . plugin_dir_url(__FILE__) . 'assets/js/expay.js"></script>';

            echo '<div>';
            if (isset($redirectUrl['key'])) {
                if ($redirectUrl['key'] == 'offline') {
                    echo '<section><a class="button alt" id="payment_button" href="' . $redirectUrl['src'] . '"> Offline payment</a></section>';
                } elseif ($redirectUrl['key'] == 'qrcode') {
                    echo '<section><a align="center">Scan QR-code with your phone</a><br><img src="' . $redirectUrl['src'] . '"></section>';
                } elseif ($redirectUrl['key'] == 'receiptUrl') {
                    echo '<section><a class="button alt" id="payment_button" href="' . $redirectUrl['src'] . '"> Download invoice </a></section>';
                } elseif ($redirectUrl['key'] == 'redirectUrl') {
                    echo '<section><a class="button alt" id="payment_button" href="' . $redirectUrl['src'] . '"> Pay via ' . $redirectUrl['name'] . ' </a></section>';
                }
            }


            echo '</div>
                  <br>
                  <a class="button alt" id="view-order-button" href="' . $order->get_view_order_url() . '"> View order information </a>
                  <a class="button cancel" id="cancel_button" href="' . $order->get_cancel_order_url() . '"> Cancel and return in cart</a>
                  </br>';
        }

        public function webhook($order_id)
        {
            $event = new ExpayEventHandler($order_id);
            $event->handle($_POST);
        }
    }

    require_once 'api/routes.php';

    function apiProcessPayment(WP_REST_Request $request)
    {
        if (!is_user_logged_in()) {
            return new WP_Error('unauthenticated', 'Unauthenticated', ['status' => 401]);
        }

        $params = $request->get_params();

        if (!$params['orderId']) {
            return new WP_Error('400', 'Invalid parameter "orderId"');
        }

        if (!$params['paymentMethodId']) {
            return new WP_Error('400', 'Invalid parameter "paymentMethodId"');
        }

        new ExpayGateway();
        $provider = new ExpayProvider();


        $order = wc_get_order($params['orderId']);

        if (!$order) {
            return new WP_Error('404', 'Order not found');
        }

        $order_amount = $order->get_total();
        $order_number = $order->get_order_number();

        $paymentMethod = null;
        $attributes = [];
        $methodList = $provider->getMethodList();

        if (is_array($methodList)) {
            foreach ($methodList as $method) {
                if ($method['id'] === intval($params['paymentMethodId'])) {
                    $paymentMethod = $method['id'];
                    if (count($method['attributes']) != 0) {
                        foreach ($method['attributes'] as $attribute) {
                            if (!empty($params['attributes'])) {
                                $attributes[$attribute['key']] = $params['attributes'][$attribute['key']];
                            }
                        }
                    }
                    break;
                }
            }
        }

        if (!$paymentMethod) {
            return new WP_Error('404', 'Payment method not found');
        }

        $order->add_order_note('Awaiting payment');

        return $provider->initPayment(
            $paymentMethod,
            $order_amount,
            $order_number,
            $order->get_view_order_url() . '?&status=success',
            $order->get_view_order_url() . '?&status=waiting',
            $order->get_view_order_url() . '?&status=reject',
            $attributes
        );
    }

    //Добавляет возможность возвращать разные размеры картинок в WooCommerce API
    add_filter("woocommerce_rest_prepare_product_object", "prepare_product_images", 10, 3);
    function prepare_product_images($response, $post, $request)
    {
        global $_wp_additional_image_sizes;

        if (empty($response->data)) {
            return $response;
        }

        foreach ($response->data['images'] as $key => $image) {
            $image_urls = [];
            foreach ($_wp_additional_image_sizes as $size => $value) {
                $image_info = wp_get_attachment_image_src($image['id'], $size);
                $response->data['images'][$key][$size] = $image_info[0];
            }
        }
        return $response;
    }
}
