<?php

defined( 'ABSPATH' ) or die( 'No direct script access allowed.' );


class ExpayProvider
{
    const SANDBOX_URL = 'https://api.sandbox.expay.asia/merchant/';
    const LIVE_URL = 'https://api.expay.asia/merchant/';

    private $url;
    private $public_key;
    private $private_key;

    public function __construct()
    {
        $this->public_key = WC()->session->get('publishable_key', '');
        $this->private_key = WC()->session->get('private_key', '');
        $this->url = WC()->session->get('testmode', '') ? self::SANDBOX_URL : self::LIVE_URL;
    }

    private function hashData($method, $params)
    {
        $string_for_hash = $method . http_build_query($params, null, '&', PHP_QUERY_RFC3986);
        return hash_hmac('sha1', $string_for_hash, $this->private_key);
    }

    public function hashResponse($response)
    {
        return hash_hmac('sha1', $response, $this->private_key);
    }

    public function getMethodList()
    {

        $getMethodsURL = $this->url . 'getMethods';
        $params = [
            'key' => $this->public_key,
            'timestamp' => time(),
        ];
        $params['hash'] = $this->hashData('getMethods?', $params);

        $ch = new Curl();
        $result = $ch->post($getMethodsURL, $params);

        if (isset($result['error']) || !empty($result['curl_error']) || isset($result['body']['error']) || $result['http_code'] === 502) {
            error_log('Expay Provider: Error response of getMethodList');// (' . $result['body']['error']['message'] ?? 'Unknown error' . ')');
            return false;
        }

        return $result['body']['response']['methods'];
    }

    public function getServicesCategories()
    {

        $getServicesURL = $this->url . "getCategories";

        $params = [
            'key' => $this->public_key,
            'timestamp' => time(),
        ];
        $params['hash'] = $this->hashData('getCategories?', $params);

        $ch = new Curl();
        $result = $ch->post($getServicesURL, $params);

        if (isset($result['error']) || !empty($result['curl_error']) || isset($result['body']['error']) || $result['http_code'] === 502) {
            error_log('Expay Provider: Error response of getServicesCategories');// (' . $result['body']['error']['message'] ?? 'Unknown error' . ')');
            return false;
        }

        $categories = [];
        foreach ($result['body']['response']['categories'] as $category) {
            $categories[$category['key']] = $category;
        }

        return $categories;
    }

    public function initPayment($service_id, $amount, $order, $url_success, $url_waiting, $url_reject, $attrs = [])
    {
        $initPaymentURL = $this->url . 'initPayment?';
        $ch = new Curl();

        $params = [
            'service_id' => $service_id,
            'amount' => $amount,
            'order' => $order,
            '_successUrl' => $url_success,
            '_waitingUrl' => $url_waiting,
            '_rejectUrl' => $url_reject,
            'key' => $this->public_key,
            'timestamp' => time()
        ];

        foreach ($attrs as $key => $value) {
            $params[$key] = $value;
        }

        $params['hash'] = $this->hashData('initPayment?', $params);

        $result = $ch->post($initPaymentURL, $params);
        if (isset($result['error']) || !empty($result['curl_error'])) {
            error_log('Expay Provider: Error response of initPayment (' . $result['body']['error']['message'] ?? 'Unknown error' . ')');
            return false;
        }

        if (array_key_exists('error', $result['body'])) {
            error_log('Expay Provider: Error response of initPayment (' . $result['body']['error']['message'] ?? 'Unknown error' . ').');
            return $result['body'];
        }

        return $result['body']['response'];
    }

    public function getPaymentStatus($payment_id)
    {
        $getStatusURL = $this->url . 'getStatus?';
        $params = [
            'payment_id' => $payment_id,
            'key' => $this->public_key,
            'timestamp' => time(),
        ];
        $params['hash'] = $this->hashData('getStatus?', $params);

        $ch = new Curl();
        $result = $ch->post($getStatusURL, $params);

        if (isset($result['error']) || !empty($result['curl_error'])) {
            error_log('Expay Provider: Error response of getPaymentStatus (' . $result['body']['error']['message'] ?? 'Unknown error' . ').');
            return false;
        }

        if (isset($result['body']['error'])) {
            error_log('Expay Provider: Error response of getPaymentStatus (' . $result['body']['error']['message'] ?? 'Unknown error' . ').');
            return false;
        }

        return $result['body']['response']['status'];
    }
}